/*jshint node:true*/

var gruntHelper = module.exports,
	grunt = require('grunt'),
	_ = require('underscore');


var Clean = require('./gruntHelpers/clean.js');
var Concat = require('./gruntHelpers/concat.js');
var Pure_grids = require('./gruntHelpers/pure_grids.js');
var Handlebars = require('./gruntHelpers/handlebars.js');
var Copy = require('./gruntHelpers/copy.js');
var Browserify = require('./gruntHelpers/browserify.js');

var configSet = {};


function getFullConfig() {

	var full = _.extend({}, configSet);
	_.each(['clean', 'concat', 'pure_grids', 'handlebars', 'copy', 'browserify'], function(key) {
		if (_.size(gruntHelper[key].config) > 0) {
			full[key] = gruntHelper[key].config;
		}
	});

	return full;

}

gruntHelper.loadTranspilerTask = function(transpilerModule) {
	transpilerModule(grunt);
};

gruntHelper.loadReworkTask = function(files){
	require('../tasks/rework.js')(grunt);

	configSet.rework = {
		default:{
			src: files
		}
	};
};

gruntHelper.loadExtractrulesvarsTask = function(){
	require('../tasks/extractrulesvars.js')(grunt);
};

gruntHelper.loadPurePushPullTask = function(){
	require('../tasks/pure_push_pull.js')(grunt);
};

gruntHelper.setGruntConfig = function(key, configuration){
	if(typeof configSet[key] === 'undefined'){
		configSet[key] = {};
	}
	_.each(configuration, function(config, configKey){
		configSet[key][configKey] = config;
	});
};

gruntHelper.runTasks = function(taskList) {

	var taskListAsKeys = {};
	_.each(taskList, function(v){
		taskListAsKeys[v.split(':')[0]] = true;
	});

	if(taskListAsKeys.clean){
		grunt.loadNpmTasks('grunt-contrib-clean');
	}
	if(taskListAsKeys.concat){
		grunt.loadNpmTasks('grunt-contrib-concat');
	}
	if(taskListAsKeys.copy){
		grunt.loadNpmTasks('grunt-contrib-copy');
	}
	if(taskListAsKeys.pure_grids){
		grunt.loadNpmTasks('grunt-pure-grids');
	}
	if(taskListAsKeys.handlebars){
		grunt.loadNpmTasks('grunt-contrib-handlebars');
	}
	if(taskListAsKeys.browserify){
		grunt.loadNpmTasks('grunt-browserify');
	}

	var gruntConfig = getFullConfig();

	if (process.env.NODE_ENV === 'test') {
		console.log('gruntConfig ->');
		console.log(taskList);
		console.log(JSON.stringify(gruntConfig, null, 4));
	}

	grunt.initConfig(gruntConfig);



	grunt.task.run(taskList);
	grunt.task.start();

};


gruntHelper.reset = function() {

	gruntHelper.clean = new Clean();
	gruntHelper.concat = new Concat();
	gruntHelper.pure_grids = new Pure_grids();
	gruntHelper.handlebars = new Handlebars();
	gruntHelper.copy = new Copy();
	gruntHelper.browserify = new Browserify();
	configSet = {};

};




gruntHelper.reset();

// expose private functions for testing
if (process.env.NODE_ENV === 'test') {
	gruntHelper._private = {
		getFullConfig: getFullConfig
	};
}
