var _ = require('underscore');

var Base = require('./base.js'),
	componentBuilder = require('../componentBuilder.js');

function Handlebars() {
	Base.call(this);
}

Handlebars.prototype = Object.create(Base.prototype);
Handlebars.prototype.constructor = Base;

Handlebars.prototype.getComponentIdFromFilePath = function(filePath) {

	var relativePath = filePath.replace(this.conf.sources.components + '/', '');
	var componentName = relativePath.split('/').pop().replace('.hbs', '');
	var namespace = relativePath.split('/').shift();
	var componentId;

	if (this.conf.sources.componentsOptions && this.conf.sources.componentsOptions[namespace] && this.conf.sources.componentsOptions[namespace].atomic) {
		componentId = [
			namespace,
			relativePath.split('/')[1],
			componentName
		].join('/');
	} else {
		componentId = namespace + '/' + componentName;
	}

	return componentId;

};

Handlebars.prototype.init = function(conf) {

	this.conf = conf;
	this.componentBuilder = componentBuilder;
	this.componentBuilder.init(conf);

};

Handlebars.prototype.add = function(key, dest, sources, options) {

	var self = this;
	options = options || {};

	var files = {};
	files[dest] = sources;

	var hbsoptions = {};

	if (options.partialsPathRegex) {

		// make them all partials
		hbsoptions.partialRegex = /\.hbs$/;

		hbsoptions.partialsPathRegex = options.partialsPathRegex;

		if (options.components) {

			hbsoptions.processPartialName = function(filePath) {

				return self.getComponentIdFromFilePath(filePath);
			};

			hbsoptions.processContent = function(content, filePath) {

				var componentId = self.getComponentIdFromFilePath(filePath),
					component = componentBuilder.getComponentById(componentId);

				if (component.dependencies.js) {

					if (content.match(/data-sife-js/m)) {
						// user specified the attribute themselves
						// simply check it's the right one
						// @TODO
					} else {

						// insert data attribute to first node
						console.log('Notice: ' + componentId + ' has JS dependency but no data-sife-js attribute found');
						console.log('         Automatic insertion');

						content = content.replace(/<([a-z0-9]+)/m, '<$1 data-sife-js="' + componentId + '"');
						console.log(content);

					}
				}

				return content;

			};

		} else if (options.themes) {
			hbsoptions.processPartialName = function(filePath) {

				var relativePath = filePath.replace(self.conf.sources.themes + '/', '');
				var partialName = filePath.split('/').pop().replace('.hbs', '');
				var themeName = relativePath.split('/').shift();

				return themeName + '/' + partialName;

			};

			hbsoptions.processName = function(filePath) {
				var relativePath = filePath.replace(self.conf.sources.themes + '/', '');

				return relativePath.replace('.hbs', '');
			};
		}


	}

	this.config[key] = {
		files: files,
		options: hbsoptions
	};

};

module.exports = Handlebars;

