/* global $,_ */
/*jshint browser:true*/
$(function() {

	String.prototype.toCamelCase = function() {
		return this.replace(/^([a-zA-Z][A-Z])|[\s-_](\w)/g, function(match, p1, p2, offset) {
			if (p2) return p2.toUpperCase();
			return p1.toLowerCase();
		});
	};

	var hrefParts = window.location.href.split('?');
	var dataSource = '';

	if (hrefParts[1].split('#').length === 2) {

		dataSource = hrefParts[1].split('#')[1];
		hrefParts[1] = hrefParts[1].split('#')[0];

	}

	var componentName,
		componentNamespace = hrefParts[1].split('/')[0];
	if (hrefParts[1].split('/').length===3) {
		componentName = hrefParts[1].split('/')[2];

	} else {
		componentName = hrefParts[1].split('/')[1];
	}


	if (!hrefParts[1]) {

		var select = '<select onchange="window.location.href=\'' + hrefParts['0'] + '?\'+this.value"><option>Choose what to render...</option>';

		for (var tplKey in window.Handlebars.partials) {
			select+='<option>' + tplKey + '</option>';
		}

		select+='</select>';
		document.body.innerHTML = select;

	} else {

		// handle modifiers
		var modifParts = hrefParts[1].split('--');
		var modifier = null;
		if(modifParts.length === 2){
			hrefParts[1] = modifParts[0];
			modifier = modifParts[1];
			componentName = componentName.split('--')[0];
		}

		// first, load the data
		var configUrl = '../components/' + hrefParts[1] + '/config.json';
		// and css
		var cssUrl = '__output__/css/components/' + hrefParts[1] + '.css';
		var css = document.createElement('link');
		css.rel = 'stylesheet';
		css.type = 'text/css';
		css.href = cssUrl;
		document.getElementsByTagName("head")[0].appendChild(css);

		$.when(
			$.getJSON(configUrl)
		).then(function(config){

			window.SIFE = window.SIFE || {styleguide:true};

			var variations = config.variations,
				skins = config.skins,
				dataSources = _.sortBy(config.data, function(ds){
					return _.keys(ds)[0] === 'standard' ? 0 : 1;
				});

			var cssPrefix = '';
			if(config.sifeConfig.sources.componentsOptions && config.sifeConfig.sources.componentsOptions[componentNamespace] && config.sifeConfig.sources.componentsOptions[componentNamespace].cssPrefix){
				cssPrefix = config.sifeConfig.sources.componentsOptions[componentNamespace].cssPrefix + '-';
			}

			if(dataSources.length===0){
				dataSources = [{'standard':{}}];
			}

			if(dataSource !== ''){
				dataSources = _.filter(dataSources, function(ds){
					console.log(_.keys(ds)[0], dataSource);
					return _.keys(ds)[0] === dataSource;
				});
			}

			function render(skin, variation, data){

				var addVariationClass = null,
					addSkinClass = null;

				if(variation){
					addVariationClass = cssPrefix+componentName.toLowerCase()+'--'+variation;
				}

				if(skin){
					addSkinClass = cssPrefix+componentName.toLowerCase()+'--s-'+skin;
				}

				// then render
				var partialName = modifier ? hrefParts[1]+'--'+modifier : hrefParts[1];
				var html = $(window.Handlebars.partials[partialName]( data[_.keys(data)[0]] ));

				if(addVariationClass){
					$(html.get(0)).addClass(addVariationClass);
				}
				if(addSkinClass){
					$(html.get(0)).addClass(addSkinClass);
				}

				if (window.Handlebars.partials[hrefParts[1] + '-wrapper']) {
					html = window.Handlebars.partials[hrefParts[1] + '-wrapper']({component: html[0].outerHTML});
				}

				if(variations.length>0 || dataSources.length>1){
					$('#target').append('<div style="font-family:monospace;background-color:#CCC;border-bottom:1px solid #FFF">'+(skin ? '['+skin+']' : '')+'.'+cssPrefix+componentName.toLowerCase()+(variation ? '--'+variation : '')+'<span style="float:right;font-size:90%;">'+_.keys(data)[0]+'&nbsp;</span></div>');
				}
				$('#target').append(html);


			}


			for (var dataSourceI = 0; dataSourceI < dataSources.length; dataSourceI++) {

				for (var skinI = -1; skinI < skins.length; skinI++) {
					render(skins[skinI], null, dataSources[dataSourceI]);
					$('#target').append('<br><br>');

					for (var i = 0; i < variations.length; i++) {

						render(skins[skinI], variations[i], dataSources[dataSourceI]);
						$('#target').append('<br><br>');
					}
				}

			}




			$.getScript('__output__/js/' + hrefParts[1] + '.js', function(){
				var scriptId = hrefParts[1].toCamelCase().split('/').join('_');
				if(window.SIFE && typeof window.SIFE[scriptId] === 'function'){
					$('[data-sife-js="'+hrefParts[1]+'"]').each(function(){
						window.SIFE[scriptId]($(this));
					});

				}
			});
		});




	}

});
