/* jshint node:true */
var utils = require('./utilities.js'),
	fs = require('fs'),
	_ = require('underscore'),
	rework = require('rework'),
	rework_vars = require('rework-vars'),
	easyimg = require('easyimage'),
	uniqid = require('uniqid'),
	Color = require('color'),
	path = require('path'),
	sifeConfig = require('../../lib/loadConfig.js')();

exports.init = function(app, express) {

	/*
	Intercept requests to theme json data file and apply utils.replaceInObject
	 */
	app.get('/themes/:themeName/data/*.json', function(request, response) {

		var themeName = request.params.themeName;

		var json = utils.tryReadJSON(request.url.substr(1));

		// extend with base standard.json
		var baseJson = utils.tryReadJSON(path.join(sifeConfig.sources.themes , themeName , 'data/standard.json'));
		json = _.extend({}, baseJson, json);


		// load config.json to put theme variables in json
		var config = utils.tryReadJSON(path.join(sifeConfig.sources.themes , themeName , 'config.json'));

		if (config.vars) {
			_.each(config.vars, function(conf, key) {
				if (typeof json[key] === 'undefined') {
					json[key] = conf.value.default;
				}
			});
		}

		utils.replaceInObject(json);
		response.end(JSON.stringify(json));

	});

	app.get('/themes/:themeName/config.json', function(request, response){
		var themeName = request.params.themeName;
		var json = utils.tryReadJSON(path.join(sifeConfig.sources.themes , themeName , 'config.json'));
		response.end(JSON.stringify(json));
	});


	app.get('/components/:componentNamespace/:atomic/:componentName/data/:jsonfile.json', function(request, response) {
		var json = utils.tryReadJSON(sifeConfig.sources.components + '/' + request.params.componentNamespace + '/' + request.params.atomic + '/' + request.params.componentName + '/data/' + request.params.jsonfile + '.json');

		// extend with base standard.json
		var baseJson = utils.tryReadJSON(sifeConfig.sources.components + '/' + request.params.componentNamespace + '/' + request.params.atomic + '/' + request.params.componentName + '/standard.json');
		json = _.extend({}, baseJson, json);

		utils.replaceInObject(json);
		response.end(JSON.stringify(json));

	});

	/*
	Intercept requests to component json data file and:
	- if using layout, use layout's json data as base
	- apply utils.replaceInObject
	 */
	app.get('/components/:componentNamespace/:componentName/data/*.json', function(request, response) {

		var json = utils.tryReadJSON(request.url.substr(1));

		/*var component = request.url.split('/components/ef/')[1].split('/')[0];
		var file = fs.readFileSync('components/ef/'+component+'/html/'+component+".hbs").toString();
		var layout = file.split('{{#extend "General/')[1];


		if(layout !== undefined){
			layout = layout.split('"}}')[0];
			var layout_json = JSON.parse(fs.readFileSync('layouts/'+ layout +'/data/standard.json'));

			json = _.extend(layout_json, json);
		}*/
		utils.replaceInObject(json);

		response.end(JSON.stringify(json));

	});

	var interceptComponentConfig = function(request, response){
		var componentPath = sifeConfig.sources.components + '/' + request.params.componentNamespace + '/' + (request.params.atomic ? request.params.atomic + '/' : '') + request.params.componentName;
		var json = utils.tryReadJSON(componentPath + '/config.json');

		var cssFiles;
		try{
			cssFiles = fs.readdirSync(componentPath + '/css/');
		}catch(err){
			cssFiles = [];
		}

		var dataFiles;
		try{
			dataFiles = fs.readdirSync(componentPath + '/data/');
		}catch(err){
			dataFiles = [];
		}
		if(dataFiles.length===0){
			dataFiles.push('standard.json');
		}
		var parentDataFile = utils.tryReadJSON(componentPath+'/standard.json');


		// add variations
		json.variations = _.filter(cssFiles, function(file){
			return file.match(/(?!.*-s-)--/);
		});
		json.variations = _.map(json.variations, function(file){
			return file.split('--')[1].replace('.css', '');
		});

		// add skins
		json.skins = _.filter(cssFiles, function(file){
			return file.match(/--s-/);
		});
		json.skins = _.map(json.skins, function(file){
			return file.split('--s-')[1].replace('.css', '');
		});

		// add data
		json.data = _.map(dataFiles, function(file){
			var name = file.replace('.json', '');
			var ret = {};
			ret[name] = utils.tryReadJSON(componentPath+'/data/'+file);

			if(parentDataFile){
				ret[name] = _.extend({}, parentDataFile, ret[name]);
			}

			return ret;
		});

		// add sife config
		json.sifeConfig = sifeConfig;


		response.end(JSON.stringify(json));
	};
	app.get('/components/:componentNamespace/:atomic/:componentName/config.json', interceptComponentConfig);
	app.get('/components/:componentNamespace/:componentName/config.json', interceptComponentConfig);



	/*
	Route for /dynamic-css-vars/
	 */
	app.get('/dynamic-css-vars/*', function(request, response) {

		var urlParts = request.url.split('/');
		var themeName = urlParts[2];
		var vars = JSON.parse(decodeURIComponent(request.url.split('?')[1]));

		var overwriteVars = ':root{';
		_.each(vars, function(val, key) {
			overwriteVars+='--' + key + ':' + val + ';';
		});
		overwriteVars+='}';

		var css = fs.readFileSync('build/base-variables.css').toString()
			+ fs.readFileSync('themes/' + themeName + '/css/variables.css').toString()
			+ overwriteVars
			+ fs.readFileSync('build/theme-' + themeName + '.css.dynamic.css').toString();

		css = rework(css)
			.use(rework_vars())
			.toString();

		response.type('css');
		response.end(css);

	});


	/*
	Route for /component-datas/foo/bar
	Responds with a list of all json files for a given component
	 */
	app.get('/component-datas/:componentNamespace/:componentName', function(request, response) {

		var files;

		try {
			files = fs.readdirSync(sifeConfig.sources.components + '/' + request.params.componentNamespace + '/' + request.params.componentName + '/data/');
		}

		catch (err) {
			files = [];
		}

		response.end(JSON.stringify(files));

	});


	/*
	Route for /component-datas/foo/baz/bar
	Responds with a list of all json files for a given component
	 */
	app.get('/component-datas/:componentNamespace/:atomic/:componentName', function(request, response) {

		var files;

		try {
			files = fs.readdirSync(sifeConfig.sources.components + '/' + request.params.componentNamespace + '/' + request.params.atomic + '/' + request.params.componentName + '/data/');
		}

		catch (err) {
			files = [];
		}

		response.end(JSON.stringify(files));

	});



	/*
	image placeholder
	placeholder/200			-> 200x200 grey bg
	placeholder/200/100		-> 200x100 grey bg
	placeholder/200/100/F00 -> 200x100 custom bg
	 */
	app.get('/placeholder/*', function(request, response) {

		var imgName = uniqid();

		var urlParts = request.url.split('/');
		urlParts.shift();
		urlParts.shift();

		var config = {
			color: '#CCC',
			width: 250,
			height: 250
		};

		if (urlParts.length === 1) {

			config.width = parseInt(urlParts[0], 10);
			config.height = parseInt(urlParts[0], 10);

		}

		else if (urlParts.length === 2) {

			config.width = parseInt(urlParts[0], 10);
			config.height = parseInt(urlParts[1], 10);

		}

		else if (urlParts.length === 3) {

			config.width = parseInt(urlParts[0], 10);
			config.height = parseInt(urlParts[1], 10);
			config.color = '#' + urlParts[2];

		}

		var color = Color(config.color);

		var execString = 'convert -size ' + config.width + 'x' + config.height + ' -background \'' + color.hexString() + '\' -fill \'' + color.darken(0.5).hexString() + '\' -font /System/Library/Fonts/Monaco.dfont -pointsize 25 -gravity center label:' + config.width + '×' + config.height + '  ' + __dirname + '/../temp/' + imgName + '.png';
		//console.log(execString);

		easyimg.exec(execString).then(
			function() {

				var img = fs.readFileSync(__dirname + '/../temp/' + imgName + '.png');
				response.writeHead(200, {'Content-Type': 'image/png'});
				response.end(img, 'binary');

				fs.unlink(__dirname + '/../temp/' + imgName + '.png');

			},
			function() {
				console.log('easyimg error');
			}
		);

	});


	app.get('/__output__/*', function(request, response){
		var urlParts = request.url.split('/'),
			outputKey = urlParts[2],
			outputPath;

		if(outputKey==='css'){
			outputPath = sifeConfig.output[outputKey][urlParts[3]];
			urlParts.shift();
			urlParts.shift();
			urlParts.shift();
			urlParts.shift();
			outputPath += '/' + urlParts.join('/');

			response.type('css');

		}else{
			urlParts.shift();
			urlParts.shift();
			urlParts.shift();

			outputPath = path.resolve(sifeConfig.output[outputKey] + '/' + urlParts.join('/')).split('?').shift();
		}

		var content;
		try{
			content = fs.readFileSync(outputPath);
		}catch(err){
			content = '/* 404 SIFE: Not found ('+outputPath+') */';
		}

		response.end(content);
	});


	/*
	For all the rest, be a simple static server
	 */
	app.use(express.static(__dirname + '/..'));// localhost/ is styleguide
	app.use(express.static(__dirname + '/../..'));// still serve files 1 level up, like bower_components

	// make sure temp/ exists
	if (!fs.statSync(__dirname + '/../temp').isDirectory()) {
		fs.mkdir(__dirname + '/../temp');
	}

};
